{{
*************************************
* Prop-BIOS Demo module
*************************************

Concept of PropBIOS services is to provide an interface that can be
used by any application that is aware of the PropBIOS services API.

This is module demonstrates using services infrastructure for "BIOS_Stdio"
Other services can be easily added following any BIOS_* API wrapper model.

Additionally, this module can be built without including/starting BIOS.
BIOS_* modules could be made available for a spin simulator for example.

Today, the BIOS code can not launch a program from SD-card. That is next.
It is very desirable at some point to make netboot possible.

This code is mostly in "draft" mode. Underlying services model may change.
}}

CON ' PinDefs
   _clkmode  = pd#_clkmode
   _xinfreq  = pd#_xinfreq

OBJ
  bios : "BIOS"                 ' includes all available drivers and is huge.
                                ' better to use a version of BIOS-select - see BIOS.spin for more info.
                                
  pd   : "PinDefs"
  keybd: "BIOS_Keyboard"        ' this is the Keyboard client API
  mouse: "BIOS_Mouse"           ' this is the Mouse client API
  fds  : "BIOS_FullDuplexSerial"' this is the FullDuplexSerial client API
  eep  : "BIOS_EEPROM"          ' Propeller's EEPROM
  sdc  : "BIOS_SDCARD"          ' SD Card driver

  bc   : "BIOS_Interface"
  stdio: "BIOS_Stdio"           ' BIOS console ... later


PUB main | hello

  bios.start    ' nothing else happens if SDBOOT or NETBOOT is specified in cfg_boot_mode!
  
  hello := string($d,"Hello World!",$d)
  stdio.print(hello)              

  ' redirect I/O to TV
  stdio.redirect(bc#BIOS_STDOUT,bc#BIOS_TV)

  stdio.printf(string($d,"stdout %04x %s\n"), p(bc.getConsole(bc#BIOS_STDOUT))+p(hello))
  stdio.setColors(@mytvpalette)

  stdio.printf(string("Hello address: $%04x "),p(@hello))
  stdio.consoleOut($c)          ' set new tile color
  stdio.consoleOut(2)           ' choose white/blue ... would be yellow / brown in standard palette  
  stdio.printf(string(" $%04x "),p(@hello))
  stdio.consoleOut($c)          ' set new tile color
  stdio.consoleOut(0)           ' choose white/black again  
  stdio.printf(string(" $%04x\n"),p(@hello))

  ' EEPROM test
  eetest
  ' SDCARD test  
  sdtest

  ' I guess including BIOS_FullDuplexSerial makes sense, but it is limited to one copy.
  ' It would be better to start multiple instances.
  ' Of course nothing stops the user from declaring multiple objects in the app.
  ' fds.start(31,30,0,115200) ' start is provided in case a user wants to change baud at runtime
  stdio.print(string($d,"Waiting for Serial port "))
  result := fds.rx
  fds.str(hello)
  
  stdio.printf(string($d,"FDS RX# "),p(result))

  ' display mouse type, x,y,z on button click
  stdio.print(string($d,"Waiting for Mouse "))
  repeat
    result := mouse.buttons
  until result <> 0
  stdio.printf(string($d,"Mouse# %d ax %d ay %d az %d\n"),p(result)+p(mouse.abs_x)+p(mouse.abs_y)+p(mouse.abs_z))
   
  ' Keyboard.spin has a startx for extended modes
  stdio.print(string($d,"Waiting for Keyboard "))
  result := keybd.getkey
  stdio.printf(string($d,"Keybd Key# %d"), p(result))

  repeat
  

pri eetest | ii, data
  eep.Initialize
  stdio.printf(string($d,"EEPROM Data[...]#",$d),0)
  repeat ii from 0 to $10-1
    data := eep.readlong(eep#EEPROM,ii)
    if(ii // 4) == 0
      stdio.printf(string($d,"%03x "),p(ii))
    stdio.printf(string("%08x "),p(data))
  putchar($d)

CON
  BIGBUF_SIZE = 255
  
VAR
   byte tbuf[20]
   byte bigbuf[BIGBUF_SIZE+1]

pri sdtest | r, sta
  stdio.print(string("Mounting.", $d))        
  sdc.mount
  stdio.print(string("Mounted.", $d))
  stdio.print(string("Dir: ", $d))
  sdc.opendir
  repeat while 0 == sdc.nextfile(@tbuf)
    stdio.printf(string("%s\n"),p(@tbuf))
  stdio.print(string("That's the dir",$d,$d))
  sta := cnt
  r := sdc.popen(string("speed.txt"), "w")
  repeat 256
     sdc.pwrite(@bigbuf, BIGBUF_SIZE)
  sdc.pclose
  r := cnt - sta
  stdio.printf(string("Writing %d bytes\n"),p(r))
  sta := cnt
  r := sdc.popen(string("speed.txt"), "r")
  repeat 256
     sdc.pread(@bigbuf, BIGBUF_SIZE)
  sdc.pclose
  r := cnt - sta
  stdio.printf(string("Reading %d bytes\n"),p(r))
  stdio.print(string("That's, all, folks! 3", $d))
  

PRI printf(fmt, arg)
{{
Print formatted output using C stdio.printf( style format. Arg list would be p(v)+p(v2)+p(v3).
Example:
stdio.printf(string("I need %d hours in a day and %d hours to %s.\n"), prm(28)+prm(10)+prm(string("sleep")));
Make a local copy of this method and the p(v) method in your code for convenience.
}}
  return stdio.printf(fmt,arg)

PRI p(v)
  return stdio.prm(v)

PRI putchar(c)
  stdio.consoleOut(c)

DAT
                        '       fore   back
                        '       color  color
mytvpalette             byte    $07,   $01    '0    white / black
                        byte    $07,   $BB    '1    white / red
                        byte    $07,   $0A    '2    white / dark blue
                        byte    $04,   $07    '3     grey / white
                        byte    $3D,   $3B    '4     cyan / dark cyan
                        byte    $6B,   $6E    '5    green / gray-green
                        byte    $BB,   $CE    '6      red / pink
                        byte    $3C,   $0A    '7     cyan / blue
  