{{
*************************************
* Prop-BIOS Constants module
*************************************
}}

CON ' Message Space

  BIOS_CMD_BASE   = $7FE0 ' long command + 7 long argument pointers
  BIOS_DBASE_ADDR = $7E00 ' config database $300 = 768 bytes need bigger ?
  BIOS_DBASE_SIZE = BIOS_CMD_BASE-BIOS_DBASE_ADDR
  
  ARG1 = 4
  ARG2 = 8
  ARG3 = 12
  ARG4 = 16
  ARG5 = 20
  ARG6 = 24
  ARG7 = 28
  
  BIOS_CMD_SHFT   = 8
  BIOS_CMD_MASK   = $ffffff00
  
  BIOS_RSVD       = 0
  BIOS_TV         = $100
  BIOS_VGA        = $200
  BIOS_FDSERIAL   = $300
  BIOS_SDCARD     = $400
  BIOS_MOUSE      = $500
  BIOS_KEYBOARD   = $600
  BIOS_EEPROM     = $700

  BIOS_STDIN      = 1
  BIOS_STDOUT     = 2
  BIOS_STDERR     = 3

  BIOS_STDIN_SER  = 1
  BIOS_STDIN_KEY  = 2
  BIOS_STDIN_DEV  = BIOS_STDIN_SER

  BIOS_STDOUT_SER = 1
  BIOS_STDOUT_TV  = 2
  BIOS_STDOUT_VGA = 3
  BIOS_STDOUT_DEV = BIOS_STDOUT_SER 
  
  BIOS_STDERR_SER = 1
  BIOS_STDERR_TV  = 2
  BIOS_STDERR_VGA = 3
  BIOS_STDERR_DEV = BIOS_STDERR_SER 
  

CON ' BIOS config data indices

  ' CRC16 is at the end in the last 2 bytes of the config database
  '
  BIOS_CFG_ADDR_CRC16 = BIOS_CMD_BASE+BIOS_DBASE_ADDR-2 

  BIOS_CFG_PINS_SIZE1 = 32 * 2 ' 32 words for 00-31
  BIOS_CFG_PINS_SIZE2 = 32 * 2 ' 32 words for 32-63

  #0 ' Header Block Definitions - word wide integers
  BIOS_CFG_HEADER_VERSION
  BIOS_CFG_HEADER_CHIPID 
  BIOS_CFG_HEADER_BOOT_MODE 
  BIOS_CFG_HEADER_BOOT_VERBOSITY
  BIOS_CFG_HEADER_LAST 

  #0 ' Statistics Block Definitions - word wide integers
  BIOS_CFG_STATS_BOOT_OK
  BIOS_CFG_STATS_BOOT_FAIL
  BIOS_CFG_STATS_CONFIG_ERRORS
  BIOS_CFG_STATS_MISC
  BIOS_CFG_STATS_LAST 
  
  #0 ' USER Block Definitions  - C strings
  BIOS_CFG_USER_ID
  BIOS_CFG_USER_PASSWD
  BIOS_CFG_USER_LICENSE
  BIOS_CFG_USER_MISC
  BIOS_CFG_USER_LAST 

  #0 ' IP Block Definitions - C strings
  BIOS_CFG_IP4_ADDR
  BIOS_CFG_IP4_NETMASK
  BIOS_CFG_IP4_GATEWAY
  BIOS_CFG_IP4_BOOTP_FILE
  BIOS_CFG_IP4_LAST 

  
CON ' BIOS_RSVD command constants
  #BIOS_RSVD
  BIOS_RSVD_NONE
  BIOS_CONFIG_INIT
  BIOS_CONSOLE_PRINT
  BIOS_CONSOLE_ERROR
  BIOS_CONSOLE_INPUT
  BIOS_CONSOLE_SETCOLORS
  BIOS_CONSOLE_OUT
    
{
Note that start/stop are missing from the enums below.
This is on purpose. Client APIs should not do start/stop.
Bios services should do start/stop instead.
There are exceptions of course like sdcard.mount,
but SDCARDs are not always inserted. 
}

CON ' TV services
  #BIOS_TV
  BIOS_TV_NONE                  ' No command requested or pending.
  BIOS_TV_BIN                   ' bin command for printing a number as binary
  BIOS_TV_SETCOLORS             ' setcolors command to change palette
  BIOS_TV_DEC                   ' dec command for printing a number as decimal
  BIOS_TV_HEX                   ' hex command for printing a number as hex
  BIOS_TV_OUT                   ' out command with standard meanings
  BIOS_TV_PRINT                 ' print command to print a non-LFCR string
  BIOS_TV_PUTS                  ' puts command to print a string + LFCR
  BIOS_TV_STR                   ' str command to print a string

CON ' VGA services
  #BIOS_VGA
  BIOS_VGA_NONE                 ' No command requested or pending.
  BIOS_VGA_BIN                  ' bin command for printing a number as binary
  BIOS_VGA_SETCOLORS            ' setcolors command to change palette
  BIOS_VGA_DEC                  ' dec command for printing a number as decimal
  BIOS_VGA_HEX                  ' hex command for printing a number as hex
  BIOS_VGA_OUT                  ' out command with standard meanings
  BIOS_VGA_PRINT                ' print command to print a non-LFCR string
  BIOS_VGA_PUTS                 ' puts command to print a string + LFCR
  BIOS_VGA_STR                  ' str command to print a string

CON ' Serial services
  #BIOS_FDSERIAL
  BIOS_FDSERIAL_NONE            ' No command requested or pending.
  BIOS_FDSERIAL_MODEBAUD        ' restart start with new mode & baud parameter
  BIOS_FDSERIAL_RXFLUSH         ' rxflush command
  BIOS_FDSERIAL_RXCHECK         ' rxcheck : rxbyte with return code in ARG1
  BIOS_FDSERIAL_RXTIME          ' rxtime(ms) : rxbyte | t
  BIOS_FDSERIAL_RX              ' rx : rxbyte
  BIOS_FDSERIAL_TX              ' tx(txbyte)
  BIOS_FDSERIAL_STR             ' str(stringptr)
  BIOS_FDSERIAL_DEC             ' dec(value) | i
  BIOS_FDSERIAL_HEX             ' hex(value, digits)
  BIOS_FDSERIAL_BIN             ' bin(value, digits)

CON ' Mouse services
  #BIOS_MOUSE
  BIOS_MOUSE_NONE               ' No command requested or pending.
  BIOS_MOUSE_PRESENT            ' present : type
  BIOS_MOUSE_BUTTON             ' button(b) : state
  BIOS_MOUSE_BUTTONS            ' buttons : states
  BIOS_MOUSE_ABS_X              ' abs_x : x
  BIOS_MOUSE_ABS_Y              ' abs_y : y
  BIOS_MOUSE_ABS_Z              ' abs_z : z
  BIOS_MOUSE_DELTA_RESET        ' delta_reset
  BIOS_MOUSE_DELTA_X            ' delta_x : x | newx
  BIOS_MOUSE_DELTA_Y            ' delta_y : y | newy
  BIOS_MOUSE_DELTA_Z            ' delta_z : z | newz
  BIOS_MOUSE_BOUND_LIMITS       ' bound_limits(xmin, ymin, zmin, xmax, ymax, zmax) | i
  BIOS_MOUSE_BOUND_SCALES       ' bound_scales(x_scale, y_scale, z_scale)
  BIOS_MOUSE_BOUND_PRESET       ' bound_preset(x, y, z) | i, d
  BIOS_MOUSE_BOUND_X            ' bound_x : x
  BIOS_MOUSE_BOUND_Y            ' bound_y : y
  BIOS_MOUSE_BOUND_Z            ' bound_z : z

CON ' Keyboard services
  #BIOS_KEYBOARD
  BIOS_KEYBOARD_NONE            ' No command requested or pending.
  BIOS_KEYBOARD_STARTX          ' startx(dpin, cpin, locks, auto) : okay
  BIOS_KEYBOARD_PRESENT         ' present : truefalse
  BIOS_KEYBOARD_KEY             ' key : keycode
  BIOS_KEYBOARD_GETKEY          ' getkey : keycode
  BIOS_KEYBOARD_NEWKEY          ' newkey : keycode
  BIOS_KEYBOARD_GOTKEY          ' gotkey : truefalse
  BIOS_KEYBOARD_CLEARKEYS       ' clearkeys
  BIOS_KEYBOARD_KEYSTATE        ' keystate(k) : state  

CON ' EEPROM Services
  #BIOS_EEPROM
  BIOS_EEPROM_NONE              ' No command requested or pending.
  BIOS_EEPROM_INIT              ' Initialize(SCL) | SDA              ' An I2C device may be left in an
  BIOS_EEPROM_WRITE             ' Write(SCL, data) : ackbit | SDA
  BIOS_EEPROM_READ              ' Read(SCL, ackbit): data | SDA
  BIOS_EEPROM_READPAGE          ' ReadPage(SCL, devSel, addrReg, dataPtr, count) : ackbit
  BIOS_EEPROM_READBYTE          ' ReadByte(SCL, devSel, addrReg) : data
  BIOS_EEPROM_READWORD          ' ReadWord(SCL, devSel, addrReg) : data
  BIOS_EEPROM_READLONG          ' ReadLong(SCL, devSel, addrReg) : data
  BIOS_EEPROM_WRITEPAGE         ' WritePage(SCL, devSel, addrReg, dataPtr, count) : ackbit
  BIOS_EEPROM_WRITEBYTE         ' WriteByte(SCL, devSel, addrReg, data)
  BIOS_EEPROM_WRITEWORD         ' WriteWord(SCL, devSel, addrReg, data)
  BIOS_EEPROM_WRITELONG         ' WriteLong(SCL, devSel, addrReg, data)
  BIOS_EEPROM_WRITEWAIT         ' WriteWait(SCL, devSel, addrReg) : ackbit  

CON ' SD Card Services
  #BIOS_SDCARD
  BIOS_SDCARD_NONE              ' No command requested or pending.
  BIOS_SDCARD_MOUNT             ' mount(basepin)
  BIOS_SDCARD_PFLUSH            ' pflush : rc
  BIOS_SDCARD_PCLOSE            ' pclose : rc
  BIOS_SDCARD_POPEN             ' popen(s, mode) : rc
  BIOS_SDCARD_PREAD             ' pread(ubuf, count) : rc
  BIOS_SDCARD_PGETC             ' pgetc : rc
  BIOS_SDCARD_PWRITE            ' pwrite(ubuf, count) : rc
  BIOS_SDCARD_PPUTC             ' pputc(c) : rc
  BIOS_SDCARD_OPENDIR           ' opendir
  BIOS_SDCARD_NEXTFILE          ' nextfile(fbuf) : rc

PUB run(cmd) : value
  long[BIOS_CMD_BASE] := cmd
  repeat while long[BIOS_CMD_BASE] <> 0
  result := long[BIOS_CMD_BASE + ARG1]

PUB set(arg, val)
  long[BIOS_CMD_BASE + arg] := val
  
DAT
  stdin  long 0
  stdout long 0
  stderr long 0
  
PUB setConsole(type, devid)
  case type
    BIOS_STDIN:  stdin  := devid
    BIOS_STDOUT: stdout := devid
    BIOS_STDERR: stderr := devid

PUB getConsole(type) : rc
  case type
    BIOS_STDIN:  result := stdin
    BIOS_STDOUT: result := stdout
    BIOS_STDERR: result := stderr
        